package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.state.EncryptionStyle;
import net.knarcraft.bookswithoutborders.state.ItemSlot;
import net.knarcraft.bookswithoutborders.utility.EncryptionHelper;
import net.knarcraft.bookswithoutborders.utility.InventoryHelper;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * Command executor for the encrypt command
 */
public class CommandEncrypt implements TabExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if (performPreChecks(sender, args, 1, "You must specify a key to encrypt a book!") == null) {
            return false;
        }

        EncryptionStyle encryptionStyle = args.length == 2 ? EncryptionStyle.getFromString(args[1]) : EncryptionStyle.SUBSTITUTION;
        return encryptBook(encryptionStyle, (Player) sender, args[0], "");
    }

    /**
     * Performs necessary pre-checks before going through with the encryption
     *
     * @param sender                <p>The sender trying to encrypt a book</p>
     * @param args                  <p>The arguments given</p>
     * @param necessaryArguments    <p>How many arguments is the minimum requirement</p>
     * @param missingArgumentsError <p>The error to show if a required argument is missing</p>
     * @return <p>The metadata of the book to encrypt, or null if any checks fail</p>
     */
    BookMeta performPreChecks(CommandSender sender, String[] args, int necessaryArguments, String missingArgumentsError) {
        if (!(sender instanceof Player player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "This command can only be used by a player!");
            return null;
        }

        if (InventoryHelper.notHoldingOneWrittenBookCheck(player,
                "You must be holding a written book to encrypt it!",
                "You cannot encrypt two books at once!")) {
            return null;
        }

        int argumentCount = args.length;
        if (argumentCount < necessaryArguments) {
            BooksWithoutBorders.sendErrorMessage(player, missingArgumentsError);
            return null;
        }
        if (argumentCount > necessaryArguments + 1) {
            BooksWithoutBorders.sendErrorMessage(player, "Too many command options specified!");
            return null;
        }

        ItemStack heldBook = InventoryHelper.getHeldBook(player, true);
        BookMeta bookMetadata = (BookMeta) heldBook.getItemMeta();
        if (bookMetadata == null) {
            BooksWithoutBorders.sendErrorMessage(player, "Your book seems to be corrupt!");
            return null;
        }
        if (!bookMetadata.hasPages()) {
            BooksWithoutBorders.sendErrorMessage(player, "Book must have contents to encrypt!");
            return null;
        }
        return bookMetadata;
    }

    /**
     * Encrypts the given book
     *
     * @param encryptionStyle <p>The encryption style to use</p>
     * @param player          <p>The player encrypting the book</p>
     * @param key             <p>The encryption key to use</p>
     * @param group           <p>The group to encrypt for</p>
     * @return <p>True if the book was encrypted successfully</p>
     */
    boolean encryptBook(EncryptionStyle encryptionStyle, Player player, String key, String group) {
        ItemSlot heldSlot = InventoryHelper.getHeldSlotBook(player, false, false, true, true);
        ItemStack encryptedBook = EncryptionHelper.encryptBook(player, heldSlot == ItemSlot.MAIN_HAND, key, encryptionStyle, group);

        if (encryptedBook != null) {
            InventoryHelper.setHeldWrittenBook(player, encryptedBook);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias, String[] args) {
        return doTabCompletion(args, false);
    }

    /**
     * Gets a list of string for tab completions
     *
     * @param args         <p>The arguments given</p>
     * @param groupEncrypt <p>Whether to auto-complete for group encryption</p>
     * @return <p>The strings to auto-complete</p>
     */
    protected List<String> doTabCompletion(String[] args, boolean groupEncrypt) {
        int argumentsCount = args.length;

        List<String> encryptionStyles = new ArrayList<>();
        encryptionStyles.add("dna");
        encryptionStyles.add("substitution");

        if (argumentsCount == 1) {
            List<String> info = new ArrayList<>();
            info.add("<password>");
            return info;
        } else if (argumentsCount == 2) {
            if (groupEncrypt) {
                List<String> info = new ArrayList<>();
                info.add("<group>");
                return info;
            } else {
                return encryptionStyles;
            }
        } else if (argumentsCount == 3 && groupEncrypt) {
            return encryptionStyles;
        }
        return new ArrayList<>();
    }

}
