package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.utility.BookLoader;
import net.knarcraft.bookswithoutborders.utility.FileHelper;
import net.knarcraft.bookswithoutborders.utility.InputCleaningHelper;
import net.knarcraft.bookswithoutborders.utility.TabCompletionHelper;
import org.bukkit.Server;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * Command executor for the give command
 */
public class CommandGive implements TabExecutor {

    private final BooksWithoutBorders booksWithoutBorders = BooksWithoutBorders.getInstance();

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if (!(sender instanceof Player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "This command can only be used by a player!");
            return false;
        }

        return giveBook(sender, args, false, "player");
    }

    /**
     * Gives a book to another player
     *
     * @param sender     <p>The sender trying to give a book</p>
     * @param args       <p>The arguments given</p>
     * @param givePublic <p>Whether to give a public book</p>
     * @param folder     <p>The folder containing the book to load</p>
     * @return <p>True if the book was given successfully</p>
     */
    boolean giveBook(CommandSender sender, String[] args, boolean givePublic, String folder) {
        if (args.length == 1 || args.length > 4) {
            BooksWithoutBorders.sendErrorMessage(sender, "Incorrect number of arguments for this command!");
            return false;
        }

        if (args.length == 0) {
            FileHelper.printBooks(sender, givePublic);
            return true;
        }

        //Organize and parse input
        String bookIdentifier = args[0];
        String receivingPlayerName = args[1];
        String copies = "1";
        String isSigned = "true";
        if (args.length == 4) {
            copies = args[2];
            isSigned = args[3];
        } else if (args.length == 3) {
            if (args[2].equalsIgnoreCase("true") || args[2].equalsIgnoreCase("false")) {
                isSigned = args[2];
            } else {
                copies = args[2];
            }
        }

        //Load books available to the player
        try {
            Integer.parseInt(bookIdentifier);
            BooksWithoutBorders.updateBooks(sender, givePublic);
        } catch (NumberFormatException ignored) {
        }

        Player receivingPlayer = booksWithoutBorders.getServer().getPlayer(receivingPlayerName);
        if (receivingPlayer == null) {
            BooksWithoutBorders.sendErrorMessage(sender, "Player not found!");
            return false;
        }

        if (receivingPlayer.getInventory().firstEmpty() == -1) {
            BooksWithoutBorders.sendErrorMessage(sender, "Receiving player must have space in their inventory to receive books!");
            return false;
        }

        String bookToLoad = InputCleaningHelper.cleanString(bookIdentifier);
        try {
            ItemStack newBook = BookLoader.loadBook(sender, bookToLoad, isSigned, folder, Integer.parseInt(copies));
            if (newBook != null) {
                receivingPlayer.getInventory().addItem(newBook);
                BooksWithoutBorders.sendSuccessMessage(sender, "Book sent!");
                BooksWithoutBorders.sendSuccessMessage(receivingPlayer, "Book received!");
                return true;
            } else {
                BooksWithoutBorders.sendErrorMessage(sender, "Book failed to load!");
                return false;
            }
        } catch (NumberFormatException e) {
            BooksWithoutBorders.sendErrorMessage(sender, "Invalid number of book copies specified!");
            return false;
        }
    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias, @NotNull String[] args) {
        return doTabCompletion(sender, args, false);
    }

    /**
     * Performs the actual tab completion
     *
     * @param sender     <p>The sender of the command</p>
     * @param args       <p>The arguments given</p>
     * @param listPublic <p>Whether to list public files or player files</p>
     * @return <p>A list of available choices</p>
     */
    protected List<String> doTabCompletion(CommandSender sender, String[] args, boolean listPublic) {
        Server server = booksWithoutBorders.getServer();

        int argumentCount = args.length;

        if (argumentCount > 2) {
            //Don't continue with autocompletion if the recipient is invalid
            if (server.getPlayer(args[1]) == null) {
                return new ArrayList<>();
            }
        }

        if (argumentCount == 1) {
            //Return list of books
            return BooksWithoutBorders.getAvailableBooks(sender, listPublic);
        } else if (argumentCount == 2) {
            //Return online players
            return null;
        } else if (argumentCount == 3) {
            //Number of copies
            return TabCompletionHelper.getBooleansAndNumbers(1, 3);
        } else if (argumentCount == 4) {
            //Signed
            try {
                Integer.parseInt(args[2]);
                return TabCompletionHelper.getBooleans();
            } catch (NumberFormatException e) {
                return new ArrayList<>();
            }
        }
        return new ArrayList<>();
    }

}
