package net.knarcraft.bookswithoutborders.listener;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.utility.BookLoader;
import net.knarcraft.bookswithoutborders.utility.InputCleaningHelper;
import net.knarcraft.bookswithoutborders.utility.InventoryHelper;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerItemHeldEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.inventory.meta.BookMeta;
import org.bukkit.inventory.meta.ItemMeta;

import java.io.File;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;

/**
 * A listener for listening to player-related events such as joining or holding a book
 */
public class PlayerEventListener implements Listener {

    private final String slash = getSlash();
    private final BooksWithoutBorders booksWithoutBorders = BooksWithoutBorders.getInstance();

    @EventHandler
    public void onHold(PlayerItemHeldEvent event) {
        if (event.isCancelled()) {
            return;
        }

        Player player = event.getPlayer();
        int selectedSlot = event.getNewSlot();
        PlayerInventory playerInventory = player.getInventory();
        ItemStack selectedItem = playerInventory.getItem(selectedSlot);

        //Ignore irrelevant items
        if (selectedItem == null || selectedItem.getType() != Material.WRITTEN_BOOK) {
            return;
        }

        ItemMeta itemMetadata = selectedItem.getItemMeta();
        if (itemMetadata == null) {
            return;
        }

        //Update the book the user is viewing
        updateBookInHand(player, itemMetadata, true);
    }

    @EventHandler
    public void onPlayerJoin(PlayerJoinEvent event) {
        Player player = event.getPlayer();

        //Handle new players
        if (!player.hasPlayedBefore()) {
            boolean sendMessage = true;

            //Gives new players necessary books
            for (String bookName : BooksWithoutBordersConfig.getFirstBooks()) {
                sendMessage = giveBookToNewPlayer(bookName, player, sendMessage);
            }
        }

        //Updates any books in either hand
        ItemStack mainHandItem = InventoryHelper.getHeldItem(player, true);
        ItemStack offHandItem = InventoryHelper.getHeldItem(player, false);
        if (mainHandItem.getType() == Material.WRITTEN_BOOK) {
            ItemMeta itemMetadata = mainHandItem.getItemMeta();
            updateBookInHand(player, itemMetadata, true);
        }
        if (offHandItem.getType() == Material.WRITTEN_BOOK) {
            ItemMeta itemMetadata = offHandItem.getItemMeta();
            updateBookInHand(player, itemMetadata, false);
        }
    }

    /**
     * Gives a book to a player joining for the first time
     *
     * @param bookName    <p>The name of the book to give</p>
     * @param player      <p>The player which just joined</p>
     * @param sendMessage <p>Whether to send a message to the joining player</p>
     * @return <p>True if a message has yet to be sent</p>
     */
    private boolean giveBookToNewPlayer(String bookName, Player player, boolean sendMessage) {
        if (!bookName.trim().isEmpty()) {

            //Give the book to the player if it exists
            ItemStack newBook = BookLoader.loadBook(player, bookName, "true", "public");
            if (newBook != null) {
                player.getInventory().addItem(newBook);
            }

            //Send the player a welcome message if it exists
            String welcomeMessage = BooksWithoutBordersConfig.getWelcomeMessage();
            if (!welcomeMessage.trim().isEmpty() && newBook != null && sendMessage) {
                sendMessage = false;
                booksWithoutBorders.getServer().getScheduler().scheduleSyncDelayedTask(booksWithoutBorders,
                        () -> player.sendMessage(welcomeMessage), 40L);
            }
        }
        return sendMessage;
    }

    /**
     * Updates a book in one of the player's hands
     *
     * @param player       <p>The player to update</p>
     * @param itemMetadata <p>Information about the held book</p>
     * @param mainHand     <p>Whether to update the book in the player's main hand</p>
     */
    private void updateBookInHand(Player player, ItemMeta itemMetadata, boolean mainHand) {
        PlayerInventory playerInventory = player.getInventory();
        ItemStack updatedBook = updateBook(player, (BookMeta) itemMetadata);
        if (updatedBook != null) {
            if (mainHand) {
                playerInventory.setItemInMainHand(updatedBook);
            } else {
                playerInventory.setItemInOffHand(updatedBook);
            }
        }
    }

    /**
     * Updates old books to a newer format
     *
     * @param player  <p>The player holding the book</p>
     * @param oldBook <p>Metadata about the held book</p>
     * @return <p>An updated book</p>
     */
    public ItemStack updateBook(Player player, BookMeta oldBook) {
        //handles hacked title-less books
        if (oldBook.getTitle() == null || oldBook.getTitle().length() < 3) {
            return null;
        }

        if (oldBook.getTitle().substring(oldBook.getTitle().length() - 3).equalsIgnoreCase("[U]")) {
            String fileName;

            if (oldBook.getAuthor() != null && oldBook.getAuthor().equalsIgnoreCase("unknown")) {
                //Unknown author is ignored
                fileName = oldBook.getTitle();
            } else {
                fileName = oldBook.getTitle() + BooksWithoutBordersConfig.getTitleAuthorSeparator() + oldBook.getAuthor();
            }

            String cleanPlayerName = InputCleaningHelper.cleanString(player.getName());

            String[] possiblePaths = new String[]{
                    getBookFolder() + fileName + ".yml",
                    getBookFolder() + fileName + ".txt",
                    getBookFolder() + cleanPlayerName + slash + fileName + ".yml",
                    getBookFolder() + cleanPlayerName + slash + fileName + ".txt"
            };

            for (String path : possiblePaths) {
                File file = new File(path);
                if (file.isFile()) {
                    return BookLoader.loadBook(player, fileName, "true", "player");
                }
            }
            return null;
        }

        return null;
    }

}
