package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.state.BookDirectory;
import org.bukkit.Material;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * A helper class for loading books from files
 */
public final class BookLoader {

    private BookLoader() {
    }

    /**
     * Loads the given book
     *
     * @param sender    <p>The command sender trying to load the book</p>
     * @param fileName  <p>The index or file name of the book to load</p>
     * @param isSigned  <p>Whether to load the book as signed, and not unsigned</p>
     * @param directory <p>The directory to save the book in</p>
     * @return <p>The loaded book</p>
     */
    public static ItemStack loadBook(CommandSender sender, String fileName, String isSigned, String directory) {
        return loadBook(sender, fileName, isSigned, directory, 1);
    }

    /**
     * Loads the given book
     *
     * @param sender    <p>The command sender trying to load the book</p>
     * @param fileName  <p>The index or file name of the book to load</p>
     * @param isSigned  <p>Whether to load the book as signed, and not unsigned</p>
     * @param directory <p>The directory to save the book in</p>
     * @param numCopies <p>The number of copies to load</p>
     * @return <p>The loaded book</p>
     */
    public static ItemStack loadBook(CommandSender sender, String fileName, String isSigned, String directory, int numCopies) {
        BookDirectory bookDirectory = BookDirectory.getFromString(directory);

        //Find the filename if a book index is given
        try {
            int bookIndex = Integer.parseInt(fileName);
            List<String> availableFiles = BooksWithoutBorders.getAvailableBooks(sender, bookDirectory == BookDirectory.PUBLIC);
            if (bookIndex <= availableFiles.size()) {
                fileName = availableFiles.get(Integer.parseInt(fileName) - 1);
            }
        } catch (NumberFormatException ignored) {
        }

        //Get the full path of the book to load
        File file = getFullPath(sender, fileName, bookDirectory, directory);
        if (file == null) {
            return null;
        }

        //Make sure the player can pay for the book
        if (BooksWithoutBordersConfig.booksHavePrice() && !sender.hasPermission("bookswithoutborders.bypassBookPrice") &&
                (bookDirectory == BookDirectory.PUBLIC || bookDirectory == BookDirectory.PLAYER) &&
                EconomyHelper.cannotPayForBookPrinting((Player) sender, numCopies)) {
            return null;
        }

        //Generate a new empty book
        ItemStack book;
        BookMeta bookMetadata = (BookMeta) BooksWithoutBorders.getItemFactory().getItemMeta(Material.WRITTEN_BOOK);
        if (isSigned.equalsIgnoreCase("true")) {
            book = new ItemStack(Material.WRITTEN_BOOK);
        } else {
            book = new ItemStack(Material.WRITABLE_BOOK);
        }

        //Load the book from the given file
        BookToFromTextHelper.bookFromFile(file, bookMetadata);
        if (bookMetadata == null) {
            BooksWithoutBorders.sendErrorMessage(sender, "File was blank!!");
            return null;
        }

        //Remove "encrypted" from the book lore
        if (bookDirectory == BookDirectory.ENCRYPTED && bookMetadata.hasLore()) {
            List<String> oldLore = bookMetadata.getLore();
            if (oldLore != null) {
                List<String> newLore = new ArrayList<>(oldLore);
                newLore.remove(0);
                bookMetadata.setLore(newLore);
            }
        }

        //Set the metadata and amount to the new book
        book.setItemMeta(bookMetadata);
        book.setAmount(numCopies);

        return book;
    }

    /**
     * Gets a File pointing to the wanted book
     *
     * @param sender        <p>The sender to send errors to</p>
     * @param fileName      <p>The name of the book file</p>
     * @param bookDirectory <p>The book directory the file resides in</p>
     * @param directory     <p>The relative directory given</p>
     * @return <p>A file or null if it does not exist</p>
     */
    private static File getFullPath(CommandSender sender, String fileName, BookDirectory bookDirectory, String directory) {
        File file = null;
        String slash = BooksWithoutBordersConfig.getSlash();
        String bookFolder = BooksWithoutBordersConfig.getBookFolder();
        if (bookDirectory == BookDirectory.PUBLIC) {
            file = FileHelper.getBookFile(bookFolder + fileName);
        } else if (bookDirectory == BookDirectory.PLAYER) {
            file = FileHelper.getBookFile(bookFolder + InputCleaningHelper.cleanString(sender.getName()) + slash + fileName);
        } else if (bookDirectory == BookDirectory.ENCRYPTED) {
            file = FileHelper.getBookFile(bookFolder + "Encrypted" + slash + directory + slash + fileName);
        }
        if (file == null || !file.isFile()) {
            BooksWithoutBorders.sendErrorMessage(sender, "Incorrect file name!");
            return null;
        } else {
            return file;
        }
    }

}
